;;; -*- Mode: lisp; Syntax: ansi-common-lisp; Base: 10 -*-
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; Copyright (c) 2000 The Regents of the University of California.
;;; All rights reserved. 
;;; 
;;; Permission is hereby granted, without written agreement and without
;;; license or royalty fees, to use, copy, modify, and distribute this
;;; software and its documentation for any purpose, provided that the
;;; above copyright notice and the following two paragraphs appear in all
;;; copies of this software.
;;; 
;;; IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
;;; FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
;;; ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
;;; THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
;;; SUCH DAMAGE.
;;;
;;; THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
;;; INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
;;; MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
;;; PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
;;; CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
;;; ENHANCEMENTS, OR MODIFICATIONS.
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; $Id: matlisp.lisp,v 1.21 2003/06/01 15:21:59 rtoy Exp $
;;;
;;; $Log: matlisp.lisp,v $
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(in-package "COMMON-LISP-USER")

(defpackage :matlisp.system
  (:use :cl
        :asdf))

(in-package :matlisp.system)

;; just warn on warnings. Some of the f2cl generated files cause problems
;; without this.  See http://article.gmane.org/gmane.lisp.cclan.general/206
(setf asdf:*compile-file-failure-behaviour* :warn)

(defclass unix-dso (module)
  ((dso-name :reader dso-name
             :initarg :dso-name)))

(defmethod asdf::input-files ((operation compile-op) (dso unix-dso))
  (mapcar #'component-pathname (module-components dso)))
  
(defclass alien-module (module) ())

(defmethod output-files ((op compile-op) (m alien-module))
  (mapcan (lambda (c)
            (output-files op c))
          (module-components m)))

(defun unix-name (pathname)
  (namestring
   (typecase pathname
     (logical-pathname (translate-logical-pathname pathname))
     (t pathname))))

(defmethod perform :after ((operation compile-op) (dso unix-dso))
  ;; simply appends the fasls using unix cat.
  (let ((dso-name (unix-name (car (output-files operation dso))))
	(done-p (operation-done-p operation dso)))

    (unless (or done-p
                (zerop
                 (run-shell-command
                  "cat ~{~S ~} > ~S"
                  (mapcar #'unix-name
                          (mapcan (lambda (c)
                                    (output-files operation c))
                                  (module-components dso)))
                  dso-name)))

      (error 'operation-error :operation operation :component dso))))

(defmethod output-files ((operation compile-op) (dso unix-dso))
  (let* ((dir (component-pathname dso))
	 (res (list
	       (make-pathname :type "fasalad"
			      :name (dso-name dso)
			      :directory  (pathname-directory dir)
			      :defaults dir))))
    res))

(defsystem :abug
    :components
  ((:unix-dso "alien code"
              :pathname ""
              :dso-name "libmatlisp"
              :components
              ((:alien-module "CPOLY"
                              :pathname ""
                              :components
                              ((:file "nothere")))))
   (:file "packages")
   (:module "foreign-interface"
            :pathname "src/"
            :depends-on ("packages" "alien code")
            :components ((:file "f77-mangling")))))

  