;;; -*- Mode: lisp -*-
(defpackage #:matlisp2-asd
  (:use :cl :asdf))

(in-package :matlisp2-asd)

(defvar *fortran-compiler* "gfortran")
(defvar *c-compiler* "gcc")
(defvar *linker* "gfortran")

;; just warn on warnings. Some of the f2cl generated files cause problems
;; without this.  See http://article.gmane.org/gmane.lisp.cclan.general/206
(setf asdf:*compile-file-failure-behaviour* :warn)

(defclass compiler-options-mixin ()
  ((compiler-options :initform nil
		     :initarg :compiler-options)))

;;;BEGIN: code to do alien-files
(defclass unix-dso (module)
  ((dso-name :reader dso-name
             :initarg :dso-name)))

(defun unix-name (pathname)
  (namestring
   (typecase pathname
     (logical-pathname (translate-logical-pathname pathname))
     (t pathname))))

;; FIXME: this needs to know about alien-modules
(defmethod asdf::input-files ((operation compile-op) (dso unix-dso))
  (mapcar #'component-pathname (module-components dso)))
;  (mapcar #'(lambda (x)
;              (funcall #'asdf::input-files operation x))
;          (module-components dso)))

  
(defmethod output-files ((operation compile-op) (dso unix-dso))
  (let ((dir (component-pathname dso)))
    (list
     (make-pathname :type "so"
                    :name (dso-name dso)  ;(car (last (pathname-directory dir)))
                    :directory  (pathname-directory dir)
                            ;; (butlast (pathname-directory dir))
                    :defaults dir))))

(defclass alien-module (module compiler-options-mixin) ())

(defmethod output-files ((op compile-op) (m alien-module))
  (mapcan (lambda (c)
            (output-files op c))
          (module-components m)))

(defclass f2cl-module (module compiler-options-mixin) ())
(defclass f77-file (cl-source-file compiler-options-mixin) ())

(defmethod compiler-options ((c compiler-options-mixin))
  (let* ((c-c-o (slot-value c 'compiler-options))
	 (p-c-o (when (typep (component-parent c) 'compiler-options-mixin)
		  (compiler-options (component-parent c)))))
    ;; this is all it takes to override options from parent:
    (append c-c-o p-c-o)))

(defmethod source-file-type ((c f77-file) (s module)) "f")

(defmethod perform ((o compile-op) (f f77-file))
  (apply #'f2cl:f2cl-compile
	 (cons (make-pathname :name (component-name f)
			      :type "f"
			      :defaults (component-pathname f))
	       (compiler-options f))))

(defsystem minpack
  :components
  ((:f2cl-module :minpack
		 :compiler-options (:include-comments t
				    :keep-lisp-file t
				    :relaxed-array-decls nil
				    :array-type :array
				    :array-slicing t
				    :package :minpack)
		 :components
		 ((:file "minpack")
		  (:f77-file "dpmpar" :depends-on ("minpack"))
		  (:f77-file "enorm" :depends-on ("minpack"))
		  (:f77-file "fdjac1" :depends-on ("dpmpar"))
		  (:f77-file "fdjac2" :depends-on ("dpmpar"))
		  (:f77-file "qrsolv" :depends-on ("minpack"))
		  (:f77-file "lmpar" :depends-on ("dpmpar" "enorm" "qrsolv"))
		  (:f77-file "qrfac" :depends-on ("dpmpar" "enorm"))
		  (:f77-file "lmdif" :depends-on ("dpmpar" "enorm" "fdjac2"
							   "lmpar" "qrfac"))
		  (:f77-file "lmdif1" :depends-on ("lmdif"))
		  (:f77-file "lmder" :depends-on ("dpmpar" "enorm" "lmpar"
							   "qrfac"))
		  (:f77-file "lmder1" :depends-on ("lmder"))
		  (:f77-file "dogleg" :depends-on ("dpmpar" "enorm"))
		  (:f77-file "qform" :depends-on ("minpack"))
		  (:f77-file "r1mpyq" :depends-on ("minpack"))
		  (:f77-file "r1updt" :depends-on ("dpmpar"))
		  (:f77-file "hybrd" :depends-on ("dogleg" "dpmpar" "enorm"
							   "fdjac1" "qform"
							   "qrfac" "r1mpyq"
							   "r1updt"))
		  (:f77-file "hybrd1" :depends-on ("hybrd"))
		  (:f77-file "hybrj" :depends-on ("dogleg" "dpmpar" "enorm"
							   "qform" "qrfac"
							   "r1mpyq" "r1updt"))
		  (:f77-file "hybrj1" :depends-on ("hybrj"))))))