;;; -*- Mode: lisp; Syntax: ansi-common-lisp; Base: 10 -*-
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; Copyright (c) 2000 The Regents of the University of California.
;;; All rights reserved. 
;;; 
;;; Permission is hereby granted, without written agreement and without
;;; license or royalty fees, to use, copy, modify, and distribute this
;;; software and its documentation for any purpose, provided that the
;;; above copyright notice and the following two paragraphs appear in all
;;; copies of this software.
;;; 
;;; IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
;;; FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
;;; ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
;;; THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
;;; SUCH DAMAGE.
;;;
;;; THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
;;; INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
;;; MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
;;; PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
;;; CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
;;; ENHANCEMENTS, OR MODIFICATIONS.
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; $Id: matlisp.lisp,v 1.21 2003/06/01 15:21:59 rtoy Exp $
;;;
;;; $Log: matlisp.lisp,v $
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(in-package "COMMON-LISP-USER")

(defpackage :matlisp.system
  (:use :cl
        :asdf))

(in-package :matlisp.system)

(defvar *fortran-compiler* "gfortran")
(defvar *c-compiler* "gcc")
(defvar *linker* "gfortran")

;; just warn on warnings. Some of the f2cl generated files cause problems
;; without this.  See http://article.gmane.org/gmane.lisp.cclan.general/206
(setf asdf:*compile-file-failure-behaviour* :warn)

;;;BEGIN: code to do alien-files
(defclass unix-dso (module)
  ((dso-name :reader dso-name
             :initarg :dso-name)))

(defun unix-name (pathname)
  (namestring
   (typecase pathname
     (logical-pathname (translate-logical-pathname pathname))
     (t pathname))))

(defmethod asdf::input-files ((operation compile-op) (dso unix-dso))
  (mapcar #'component-pathname (module-components dso)))
  
(defmethod output-files ((operation compile-op) (dso unix-dso))
  (let* ((dir (component-pathname dso))
	(res (list
	      (make-pathname :type "so"
			     :name (dso-name dso)
			     :directory  (pathname-directory dir)
			     :defaults dir))))
    res))

(defclass alien-module (module) ())

(defmethod output-files ((op compile-op) (m alien-module))
  (mapcan (lambda (c)
            (output-files op c))
          (module-components m)))

(defmethod perform :after ((operation compile-op) (dso unix-dso))
  (let ((dso-name (unix-name (car (output-files operation dso))))
	(done-p (operation-done-p operation dso)))

    (unless (or done-p
                (zerop
                 (run-shell-command
                  "~A ~A -o ~S ~{~S ~} ~A"
                  *linker*
		  "-shared"
                  dso-name
                  (mapcar #'unix-name
                          (mapcan (lambda (c)
                                    (output-files operation c))
                                  (module-components dso)))
                  "")))
      (error 'operation-error :operation operation :component dso))))

(defmethod perform ((o load-op) (c unix-dso))
  (let ((co (make-instance 'compile-op)))
    (let ((filename (car (output-files co c))))
      (sb-alien:load-shared-object filename))))

(defclass fortran-source-file (source-file) ())
(defmethod asdf:source-file-type ((f fortran-source-file) (s module)) "f")

(defmethod output-files ((op compile-op) (f fortran-source-file))
  (list
   (make-pathname :type "o" :defaults
                  (component-pathname f))))

(defmethod perform ((op compile-op) (f fortran-source-file))
  (unless
      (= 0 (run-shell-command "~A ~A -o ~S -c ~S"
                              *fortran-compiler*
                              "-fPIC -O3"
                              (unix-name (car (output-files op f)))
                              (unix-name (component-pathname f))))
    (error 'operation-error :operation op :component f)))

(defmethod perform ((operation load-op) (f fortran-source-file))
  t)

(defclass brittle-fortran-source-file (source-file) ())

(defmethod asdf:source-file-type ((f brittle-fortran-source-file) (s module)) "f")

(defmethod output-files ((op compile-op) (f brittle-fortran-source-file))
  (list
   (make-pathname :type "o" :defaults
                  (component-pathname f))))

(defmethod perform ((op compile-op) (f brittle-fortran-source-file))
  (unless
      (= 0 (run-shell-command "~A ~A -o ~S -c ~S"
                              *fortran-compiler*
                              "-fPIC -O0"
                              (unix-name (car (output-files op f)))
                              (unix-name (component-pathname f))))
    (error 'operation-error :operation op :component f)))

(defmethod perform ((operation load-op) (f brittle-fortran-source-file))
  t)

;;;END: code to do alien-files


;; This is for macros.l which has a nonstandard suffix.
(defclass cl-source-file* (cl-source-file) ())
(defmethod asdf:source-file-type ((c cl-source-file*) (s module)) "l")

(defsystem :matlispbug
    :components
  ((:unix-dso "alien code"
              :pathname ""
              :dso-name "libmatlisp"
              :components
              ((:alien-module "CPOLY"
                              :pathname "lib-src/cpoly/"
                              :components
                              ((:fortran-source-file "cpoly")))))
   (:file "packages")
   (:module "foreign-interface"
            :pathname "src/"
            :depends-on ("packages" "alien code")
            :components ((:file "f77-mangling")
                         (:file "ffi-sbcl")))))
  