(defpackage :minimatlisp.system
  (:use :cl
        :asdf))

(in-package :minimatlisp.system)

(defvar *shared-loaded* nil)

;; Takes no time, can be done over and over and over.
(eval-when (:compile-toplevel :load-toplevel :execute)
  (unless *shared-loaded*
    (sb-alien:load-shared-object "/usr/lib/libblas.so")
    (sb-alien:load-shared-object "/usr/lib/liblapack.so")
    (setf *shared-loaded* t)))

(defsystem :minimatlisp
    :components
  ((:file "packages")
   (:module "foreign-interface"
            :pathname "src/"
            :depends-on ("packages" )
            :components ((:file "f77-mangling")
                         #+:cmu (:file "ffi-cmu")
                         #+:sbcl (:file "ffi-sbcl")
                         #+:allegro (:file "ffi-acl")
                         ))
   (:module "foreign-functions"
            :pathname "src/"
            :depends-on ("foreign-interface" )
            :components ((:file "blas")
                         (:file "lapack")))
   (:module "matlisp-essentials"
            :pathname "src/"
            :depends-on ("foreign-interface" 
                         "foreign-functions"
                         )
            :components ((:file "conditions")
                         (:file "matrix")
                         (:file "ref")
                         (:file "print")
                         (:file "copy")))
   (:module "matlisp-blas-wrappers"
            :pathname "src/"
            :depends-on ("foreign-interface" 
                         "foreign-functions"
                         "matlisp-essentials"
                         )
            :components ((:file "axpy")
                         (:file "scal")
                         (:file "swap")
                         (:file "gemm")))
   
   (:module "matlisp-lapack-wrappers"
            :pathname "src/"
            :depends-on ("foreign-interface" 
                         "foreign-functions"
                         "matlisp-essentials"
                         )
            :components ((:file "gesv")
                         (:file "geev")
                         (:file "getrf")
                         (:file "getrs")
			 (:file "trsv")))
   
   (:module "matlisp-functions"
            :pathname "src/"
            :depends-on ("foreign-interface"
                         "foreign-functions"
                         "matlisp-essentials"
                         "matlisp-blas-wrappers"
                         "matlisp-lapack-wrappers"
                         )
            :components ((:file "compat")
                         (:file "help")
                         (:file "diag")
                         (:file "special")
                         (:file "reader")
                         (:file "trans")
                         (:file "realimag")
                         (:file "reshape")
                         (:file "join")
                         (:file "svd")
                         (:file "sum")
                         (:file "norm")
                         (:file "dot")
                         (:file "trace")
                         (:file "seq")
                         (:file "vec")
                         (:file "map")
                         (:file "mplus")
                         (:file "mminus")
                         (:file "mtimes")
                         (:file "mdivide")
                         (:file "msqrt")
                         #-:mswindows (:file "fft")
                         (:file "geqr")))
   (:module "special-functions"
            :pathname "src/"
            :depends-on ("matlisp-functions" )
            :components
            ((:file "specfun")))))