(in-package "MATLISP")

(defmethod trsv! ((a real-matrix) (b real-matrix) &key (upper-t t) transpose diag-has-only-ones)
   "
  Syntax
  ======
  (TRSV! a v [:upper-t t] [:transpose NIL] [:diag-has-only-ones NIL])

  Purpose
  =======
  Solves a system of linear equations
     A*x=b or A'*x=b
  with a triangular N-by-N matrix A. Usefull if A is has been obtained by a QR or LU
  factorization.

  upper-t tells if the matrix is upper triangular (the default) or not.

  transpose tells wether A*x=b should be solved (the default) or if
  rather A'*x=b is the question.

  diag-has-only-ones signalizes that the diagonals all have value 1.

  Return Value
  ============
  [1] The solution of A*x=b.
"

  (let* ((n (nrows a))
	 (m (ncols a))
	 (m2 (nrows b)))

    (assert (= m n m2))

    (make-real-matrix
     (lapack::dtrsv
      (if upper-t "u" "l")
      (if transpose "t" "n")
      (if diag-has-only-ones "u" "n")
      n
      (store a)
      n
      (store b)
      1))))

(defmethod trsv ((a real-matrix) (b real-matrix) &key (upper-t t) transpose diag-has-only-ones)
  "See documentation for trsv!"
  (trsv! a (copy b)
	 :upper-t upper-t :transpose transpose :diag-has-only-ones diag-has-only-ones))